@@ -1,3 +1,7 @@
+{-# LANGUAGE ExistentialQuantification, MultiParamTypeClasses
+  , FunctionalDependencies, Rank2Types
+  , FlexibleInstances, UndecidableInstances
+  , TypeSynonymInstances, FlexibleContexts, ScopedTypeVariables #-}
 -----------------------------------------------------------
 -- |
 -- Module      :  Database
@@ -16,31 +20,31 @@
 -- 
 -----------------------------------------------------------
 module Database.HaskellDB.Database ( 
-		-- * Operators
-	      	(!.)
+                -- * Operators
+                (!.)
 		-- * Type declarations
 		, Database(..)
 		, GetRec(..), GetInstances(..)
-
+                , GetValue(..)
 		-- * Function declarations
 		, query
 		, insert, delete, update, insertQuery
-		, tables, describe, transaction
+		, tables, describe, transaction, commit
 		, createDB, createTable, dropDB, dropTable
 		) where
 
-import Database.HaskellDB.HDBRec
 import Database.HaskellDB.FieldType
 import Database.HaskellDB.PrimQuery
 import Database.HaskellDB.Optimize (optimize, optimizeCriteria)
 import Database.HaskellDB.Query	
 import Database.HaskellDB.BoundedString
 import Database.HaskellDB.BoundedList
+import Database.HaskellDB.HDBRec
 
 import System.Time
 import Control.Monad
 
-infix 9 !. 
+infix 9 !.
 
 -- | The (!.) operator selects over returned records from
 --   the database (= rows)
@@ -72,6 +76,7 @@ data Database
 	  , dbCreateTable :: TableName -> [(Attribute,FieldDesc)] -> IO ()
 	  , dbDropDB :: String -> IO ()
 	  , dbDropTable :: TableName -> IO ()
+          , dbCommit :: IO ()
   	  }
 
 
@@ -103,21 +108,20 @@ data GetInstances s =
 
 class GetRec er vr | er -> vr, vr -> er where
     -- | Create a result record.
-    getRec :: GetInstances s -- ^ Driver functions for getting values
-			     --   of different types.
-	   -> Rel er       -- ^ Phantom argument to the the return type right
+    getRec :: GetInstances s  -- ^ Driver functions for getting values
+			      --   of different types.
+	   -> Rel er -- ^ Phantom argument to the the return type right
 	   -> Scheme       -- ^ Fields to get.
 	   -> s            -- ^ Driver-specific result data 
 	                   --   (for example a Statement object)
            -> IO (Record vr)        -- ^ Result record.
 
 instance GetRec RecNil RecNil where
-    -- NOTE: we accept extra fields, since the hacks in Optimzie could add fields that we don't want
+    -- NOTE: we accept extra fields, since the hacks in Optimize could add fields that we don't want
     getRec _ _ _ _ = return emptyRecord
 
-instance (GetValue a, GetRec er vr) 
+instance (GetValue a, GetRec er vr)
     => GetRec (RecCons f (Expr a) er) (RecCons f a vr) where
-
     getRec _ _ [] _ = fail $ "Wanted non-empty record, but scheme is empty"
     getRec vfs c (f:fs) stmt = 
 	do
@@ -177,14 +181,14 @@ insertQuery db (Table name assoc) q
 
 -- | Inserts a record into a table
 insert :: (ToPrimExprs r, ShowRecRow r, InsertRec r er) => Database -> Table er -> Record r -> IO ()
-insert db (Table name assoc) newrec	
-	= dbInsert db name (zip (attrs assoc) (exprs newrec))
-	where
-	  attrs   = map (\(attr,AttrExpr name) -> name)
-	  
+insert db (Table name assoc) newrec
+      = dbInsert db name (zip (attrs assoc) (exprs newrec))
+      where
+        attrs   = map (\(attr,AttrExpr name) -> name)
+
 -- | deletes a bunch of records	  
-delete :: ShowRecRow r => 
-	  Database -- ^ The database
+delete :: ShowRecRow r =>
+        Database -- ^ The database
        -> Table r -- ^ The table to delete records from
        -> (Rel r -> Expr Bool) -- ^ Predicate used to select records to delete
        -> IO ()
@@ -196,7 +200,7 @@ delete db (Table name assoc) criteria = dbDelete db name cs
 	  
 -- | Updates records
 update :: (ShowLabels s, ToPrimExprs s) =>
-	  Database             -- ^ The database
+        Database             -- ^ The database
        -> Table r              -- ^ The table to update
        -> (Rel r -> Expr Bool) -- ^ Predicate used to select records to update
        -> (Rel r -> Record s)  -- ^ Function used to modify selected records
@@ -236,6 +240,11 @@ transaction :: Database -- ^ Database
 	    -> IO a 
 transaction = dbTransaction
 
+-- | Commit any pending data to the database.
+commit :: Database -- ^ Database
+       -> IO ()
+commit = dbCommit
+
 -----------------------------------------------------------
 -- Functions that edit the database layout
 -----------------------------------------------------------
