@@ -37,6 +37,7 @@ module Database.HaskellDB.Sql.Default (
                                         defaultSqlExpr,
                                         defaultSqlLiteral,
                                         defaultSqlType,
+                                        defaultSqlQuote,
 
                                         -- * Utilities
                                         toSqlSelect
@@ -50,6 +51,9 @@ import Database.HaskellDB.Sql.Generate
 
 import System.Locale
 import System.Time
+import Data.Maybe (catMaybes)
+import Data.List (nubBy)
+import qualified Data.Map as Map (fromList, lookup)
 
 mkSqlGenerator :: SqlGenerator -> SqlGenerator
 mkSqlGenerator gen = SqlGenerator 
@@ -74,7 +78,8 @@ mkSqlGenerator gen = SqlGenerator
 
      sqlExpr        = defaultSqlExpr        gen,
      sqlLiteral     = defaultSqlLiteral     gen,
-     sqlType        = defaultSqlType        gen
+     sqlType        = defaultSqlType        gen,
+     sqlQuote       = defaultSqlQuote       gen
     }
 
 defaultSqlGenerator :: SqlGenerator
@@ -92,7 +97,7 @@ defaultSqlType _ t =
       IntegerT      -> SqlType "bigint"
       DoubleT       -> SqlType "double precision"
       BoolT         -> SqlType "bit"
-      CalendarTimeT -> SqlType "timestamp"
+      CalendarTimeT -> SqlType "timestamp with time zone"
       BStrT a       -> SqlType1 "varchar" a
 
 -----------------------------------------------------------
@@ -102,13 +107,13 @@ defaultSqlType _ t =
 -- | Creates a 'SqlSelect' based on the 'PrimQuery' supplied.
 -- Corresponds to the SQL statement SELECT.
 defaultSqlQuery :: SqlGenerator -> PrimQuery -> SqlSelect
-defaultSqlQuery gen = foldPrimQuery (sqlEmpty gen, 
+defaultSqlQuery gen query = foldPrimQuery (sqlEmpty gen, 
                                      sqlTable gen,
                                      sqlProject gen,
                                      sqlRestrict gen,
                                      sqlBinary gen,
                                      sqlGroup gen,
-                                     sqlSpecial gen)
+                                     sqlSpecial gen) query
 
 defaultSqlEmpty :: SqlGenerator -> SqlSelect
 defaultSqlEmpty _ = SqlEmpty
@@ -118,20 +123,114 @@ defaultSqlTable _ name schema = SqlTable name
 
 defaultSqlProject :: SqlGenerator -> Assoc -> SqlSelect -> SqlSelect
 defaultSqlProject gen assoc q
-          	| hasAggr    = select { groupby = toSqlAssoc gen nonAggrs }
-          	| otherwise  = select 
-                where
-                  select   = sql { attrs = toSqlAssoc gen assoc }
-                  sql      = toSqlSelect q
-                  hasAggr  = (not . null  . filter (isAggregate . snd)) assoc
-                  nonAggrs = filter (not . isAggregate . snd) assoc
+  -- This mess ensures we do not create another layer of SELECT when
+  -- dealing with GROUP BY phrases. If the select being built is a
+  -- real select (not a table or binary operation) and all columns to
+  -- be projected are just attributes (i.e., they copy column names
+  -- but do no computation), then we do not need to create another
+  -- layer of SELECT. We will re-use the existing select.
+  | all isAttr assoc && validSelect q = 
+    let groupables = case groupableSqlColumns . attrs $ q of
+                       [] -> Nothing
+                       gs -> Just (Columns gs)
+        -- Looks at SqlSelect columns and determines if they need to
+        -- be grouped. Not the sames as groupableProjects because this
+        -- operates on values from a SqlSelect, not PrimQuery.
+        groupableSqlColumns :: [(SqlColumn,SqlExpr)] -> [(SqlColumn,SqlExpr)]
+        groupableSqlColumns = filter groupable
+          where
+            id2 _ t = t
+            const2 t _ _ = t
+            -- determine if a sql expression should be 
+            -- placed in a group by clause. Only columns, non-aggregate
+            -- functions and expressions involving either are
+            -- groupable. Constants are not groupable. If an expression
+            -- contains any groupable values, then whole expression is groupable.
+            groupable (col, expr) = foldSqlExpr (const True -- column
+                                                , (\ _ left right -> left || right) -- binary
+                                                , const2 False  -- PrefixSqlExpr 
+                                                , const2 False -- PostfixSqlExpr
+                                                , const2 True -- FunSqlExpr
+                                                , const2 False -- AggrFunSqlExpr
+                                                , const False -- ConstSqlExpr
+                                                , (\cs e -> and (map (uncurry (||)) cs) || e) --  CaseSqlExpr
+                                                , and -- ListSqlExpr
+                                                , const False -- ExistsSqlExpr
+                                                , const2 False -- ParamSqlExpr
+                                                , False -- PlaceHolderSqlExpr
+                                                , id -- ParensSqlExpr
+                                                , id2 {- CastSqlExpr -}) expr
+        -- Rename projected columns in 
+        -- a select. Since we did not create another
+        -- layer of SELECT, we have to propogate the association list
+        -- provided into the current query, or it will not create columns
+        -- with the right names. We only go one level - no need to recursively
+        -- descend into all queries luckily.
+        subst :: Assoc -> SqlSelect -> SqlSelect
+        subst outer query@(SqlSelect { attrs = cols
+                                     , criteria = crits
+                                     , groupby = gru
+                                     , orderby = order  }) =
+              -- map attributes to their aliased columns.
+          let colToAliases = Map.fromList [(column, alias) | (alias, AttrExpr column) <- outer] 
+              getAlias column = case Map.lookup column colToAliases of
+                                  Just alias -> alias
+                                  _ -> column
+              substExpr = foldSqlExpr (ColumnSqlExpr . getAlias, BinSqlExpr, PrefixSqlExpr, PostfixSqlExpr
+                                      , FunSqlExpr, AggrFunSqlExpr, ConstSqlExpr, CaseSqlExpr
+                                      , ListSqlExpr, ExistsSqlExpr, ParamSqlExpr, PlaceHolderSqlExpr
+                                      , ParensSqlExpr,CastSqlExpr)
+              substGroup (Just (Columns cols)) = Just . Columns . map (\(col, expr) -> (getAlias col, expr)) $ cols
+              substGroup g = g
+              -- replace attributes with alias from outer query 
+          in query { attrs = map (\(currCol, expr) -> (getAlias currCol, expr)) cols
+                   , criteria = map substExpr crits 
+                   , groupby = substGroup gru
+                   , orderby = map (\(expr, ord) -> (substExpr expr, ord)) order } 
+    in subst assoc (if hasGroupMark q 
+                    -- A groupMark indicates the select wants to group
+                    -- on "all" columns. We replace the mark with the
+                    -- list of groupable columns. 
+                    then q { groupby = groupables }
+                    -- Otherwise, we just re-use the query without
+                    -- changing it (modulo substitutions).
+                    else q)
+  | hasAggr assoc || hasGroupMark newSelect = 
+    let g = groupByColumns assoc newSelect
+    in if null g
+       then newSelect { groupby = Nothing }
+       else newSelect { groupby = Just (Columns g) }
+  | otherwise  = newSelect 
+  where
+    newSelect   = (toSqlSelect q) { attrs = toSqlAssoc gen assoc }
+    hasAggr  = not . null  . filter (isAggregate . snd) 
+
+    isAttr (_, AttrExpr _) = True
+    isAttr _ = False
+
+    validSelect SqlSelect { attrs = (_:_) } = True
+    validSelect _ = False
+
+    hasGroupMark (SqlSelect { groupby = Just All }) = True
+    hasGroupMark _ = False
+
+    groupByColumns assoc sql = toSqlAssoc gen (groupableProjections assoc) ++ groupableOrderCols sql 
+    -- Find projected columns that are not constants or aggregates.
+    groupableProjections assoc = filter (not . (\x -> isAggregate x || isConstant x) . snd) assoc
+    -- Get list of order by columns which do not appear in
+    -- projected non-aggregate columns already, if any.
+    groupableOrderCols sql =
+      let eligible = filter (\x -> case x of
+                                     (ColumnSqlExpr attr) -> True
+                                     _ -> False) 
+      in [(s, e) | e@(ColumnSqlExpr s) <- eligible . map fst $ orderby sql]
 
 
--- | Takes all non-aggregate expressions in the select and adds them to
--- the 'group by' clause.
+-- | Ensures the groupby value on the SqlSelect either preserves existing 
+-- grouping or that it will group on all columns (i.e, Mark == All).
 defaultSqlGroup :: SqlGenerator -> Assoc -> SqlSelect -> SqlSelect
-defaultSqlGroup gen cols select = select { groupby = toSqlAssoc gen cols }
-    
+defaultSqlGroup _ _ q@(SqlSelect { groupby = Nothing }) = q { groupby = Just All }
+defaultSqlGroup _ _ q = q 
 
 defaultSqlRestrict :: SqlGenerator -> PrimExpr -> SqlSelect -> SqlSelect
 defaultSqlRestrict gen expr q
@@ -140,12 +239,13 @@ defaultSqlRestrict gen expr q
                   sql   = toSqlSelect q
 
 defaultSqlBinary :: SqlGenerator -> RelOp -> SqlSelect -> SqlSelect -> SqlSelect
-defaultSqlBinary _ Times q1 q2  
+defaultSqlBinary _ Times q1@(SqlSelect { }) q2@(SqlSelect { }) 
           	| null (attrs q1) = addTable q1 q2
           	| null (attrs q2) = addTable q2 q1
           	| otherwise       = newSelect { tables = [("",q1),("",q2)] }
           	where
           	  addTable sql q  = sql{ tables = tables sql ++ [("",q)] }
+defaultSqlBinary _ Times q1 q2 = newSelect { tables = [("", q1), ("", q2)] }
 defaultSqlBinary _ op q1 q2         
           	= SqlBin (toSqlOp op) q1 q2
 
@@ -173,13 +273,41 @@ toSqlOrder gen (OrderExpr o e) = (sqlExpr gen e, o')
                  OpAsc  -> SqlAsc
                  OpDesc -> SqlDesc
 
+-- | Make sure our SqlSelect statement is really a SqlSelect and not
+-- another constructor.
 toSqlSelect :: SqlSelect -> SqlSelect
 toSqlSelect sql = case sql of
-                    (SqlEmpty)          -> newSelect
-                    (SqlTable name)     -> newSelect { tables = [("",sql)] }
-                    (SqlBin op q1 q2)   -> newSelect { tables = [("",sql)] }
-                    s | null (attrs s) -> sql
-                      | otherwise  -> newSelect { tables = [("",sql)] }
+                    SqlEmpty -> newSelect
+                    SqlTable name -> newSelect { tables = [("",sql)] }
+                    -- Below we make sure to bring groupby marks that have not 
+                    -- been processed up the tree. The mark moves up the tree
+                    -- for efficiency. A "Columns" mark does not move -- it indicates
+                    -- a select that will use a group by. An All mark does move, as it
+                    -- needs to find its containing projection. Marks that move are
+                    -- replaced by Nothing.
+                    SqlBin _ _ _ -> 
+                      let (prevGroup, newSql) = findGroup sql
+                          findGroup (SqlBin op q1 q2) = 
+                            let (g1, q1') = findGroup q1
+                                (g2, q2') = findGroup q2
+                            in (g1 `or` g2, SqlBin op q1' q2')
+                          findGroup q@(SqlSelect { groupby = Just (Columns _) }) = (Nothing, q)
+                          findGroup q@(SqlSelect { groupby = Just All }) = (Just All, q { groupby = Nothing })
+                          findGroup s = (Nothing, s)
+                          or l r = maybe r Just l
+                      in newSelect { tables = [("", newSql)]
+                                   , groupby = prevGroup }
+                    SqlSelect { attrs = [] } -> sql
+                    -- Here we have a mark that should not move. 
+                    SqlSelect { groupby = Just (Columns _)} ->
+                      newSelect { tables = [("", sql)] }
+                    -- Any mark here should be moved. Notice we set the
+                    -- previous mark with Nothing (though it may already be
+                    -- Nothing).
+                    SqlSelect { groupby = group } ->
+                      newSelect { tables = [("", sql { groupby = Nothing})]
+                                , groupby = group }
+                      
 
 toSqlAssoc :: SqlGenerator -> Assoc -> [(SqlColumn,SqlExpr)]
 toSqlAssoc gen = map (\(attr,expr) -> (attr, sqlExpr gen expr))
@@ -287,7 +415,25 @@ defaultSqlExpr :: SqlGenerator -> PrimExpr -> SqlExpr
 defaultSqlExpr gen e = 
     case e of
       AttrExpr a       -> ColumnSqlExpr a
-      BinExpr op e1 e2 -> BinSqlExpr (showBinOp op) (sqlExpr gen e1) (sqlExpr gen e2)
+      BinExpr op e1 e2 ->
+        let leftE = sqlExpr gen e1
+            rightE = sqlExpr gen e2
+            paren = ParensSqlExpr
+            (expL, expR) = case (op, e1, e2) of
+              (OpAnd, e1@(BinExpr OpOr _ _), e2@(BinExpr OpOr _ _)) ->
+                (paren leftE, paren rightE)
+              (OpOr, e1@(BinExpr OpAnd _ _), e2@(BinExpr OpAnd _ _)) ->
+                (paren leftE, paren rightE)
+              (OpAnd, e1@(BinExpr OpOr _ _), e2) ->
+                (paren leftE, rightE)
+              (OpAnd, e1, e2@(BinExpr OpOr _ _)) ->
+                (leftE, paren rightE)
+              (OpOr, e1@(BinExpr OpAnd _ _), e2) ->
+                (paren leftE, rightE)
+              (OpOr, e1, e2@(BinExpr OpAnd _ _)) ->
+                (leftE, paren rightE)
+              _ -> (leftE, rightE)
+        in BinSqlExpr (showBinOp op) expL expR
       UnExpr op e      -> let (op',t) = sqlUnOp op
                               e' = sqlExpr gen e
                            in case t of
@@ -296,12 +442,15 @@ defaultSqlExpr gen e =
                                 UnOpPostfix -> PostfixSqlExpr op' e'
       AggrExpr op e    -> let op' = showAggrOp op
                               e' = sqlExpr gen e
-                           in FunSqlExpr op' [e']
+                           in AggrFunSqlExpr op' [e']
       ConstExpr l      -> ConstSqlExpr (sqlLiteral gen l)
       CaseExpr cs e    -> let cs' = [(sqlExpr gen c, sqlExpr gen x)| (c,x) <- cs] 
                               e'  = sqlExpr gen e
                            in CaseSqlExpr cs' e'
       ListExpr es      -> ListSqlExpr (map (sqlExpr gen) es)
+      ParamExpr n v    -> ParamSqlExpr n PlaceHolderSqlExpr
+      FunExpr n exprs  -> FunSqlExpr n (map (sqlExpr gen) exprs)
+      CastExpr typ e1 -> CastSqlExpr typ (sqlExpr gen e1)
 
 showBinOp :: BinOp -> String
 showBinOp  OpEq         = "=" 
@@ -366,6 +515,9 @@ defaultSqlLiteral gen l =
       OtherLit l    -> l
 
 
+defaultSqlQuote :: SqlGenerator -> String -> String
+defaultSqlQuote gen s = quote s
+
 -- | Quote a string and escape characters that need escaping
 --   FIXME: this is backend dependent
 quote :: String -> String 
